'''
  $Id: pba_mvn_pomparser.py 158 2021-02-18 20:35:39Z cl_yubikey_2018a $
  
  Copyleft ITEG IT-Engineers GmbH 2022
'''

import getopt
#import glob
#import io
#import os
import os.path
#import subprocess
import sys

from clazzes.pba import dto
from clazzes.pba import pbalogging

log = pbalogging.getLogger()


def describeUsage(argv0):
    usage = ""
    usage += "Usage: %s [options...] [pom.xml]\n" % argv0
    usage += "\n"
    usage += "Parses the specified pom.xml (or ./pom.xml if no pom.xml was specified)\n"
    usage += "and prints out some parsed data.\n"
    usage += "By default the \"maven coordinates\" triple (and name) is printed:\n"
    usage += "groupId, ArtifactId, version, and an eventual name\n"
    usage += "\n"
    usage += "After that eventual parent coordinates and modules are printed.\n"
    usage += pbalogging.getCommonUsageOptions()
    usage += "  Specific options are:\n"
    usage += "    -c   Print one information only, in CSV format, separated with ','\n"
    usage += "         If standing alone only maven coordinates are printed.\n"
    usage += "         If combined with -p only parent coordinates are printed.\n"
    usage += "         If combined with -m only modules' coordinates are printed.\n"
    usage += "    -f   Print main ('flat') coordinates only, no parent or modules.\n"
    usage += "    -m   Print module's coordinates only.\n"
    usage += "    -p   Print parent's maven coordinates only (can be combined with '-c').\n"
    usage += "    -s   Print in short text format.\n"
    usage += "\n"
    usage += "For automatic processing always use -c', '-c -p', or '-c -m'.\n"
    usage += "It's recommended to use '-q' and '2>/dev/null' too.\n"
    usage += "\n"
    return usage


def run_pba_mvn_pomparser(argv0,args):
    
    try:
        opts, pomXmlPaths = getopt.getopt(args, "cdfhmpqstv", ["help"])
    except getopt.GetoptError as err:
        print(err, file=sys.stderr) # will print something like "option -a not recognized"
        print(describeUsage(argv0), file=sys.stderr)
        sys.exit(64)

    pbaConfiguration = dto.PBAConfiguration()

    listFormat = "text"
    printParentOnly = False
    printModulesOnly = False
    printMainOnly = False

    for o, a in opts:
        if o in ["-d", "-q", "-t", "-v"]:
            continue
        elif o in ["-h" , "--help"]:
            print(describeUsage(argv0))
            sys.exit(0)
        elif o in ["-c"]:
            listFormat = "csv"
        elif o in ["-f"]:
            printMainOnly = True
        elif o in ["-m"]:
            printModulesOnly = True
        elif o in ["-p"]:
            printParentOnly = True
        elif o in ["-s"]:
            listFormat = "txt"
        else:
            log.warning("Unhandled option: %s %s" %
                  (o, a))

    if pomXmlPaths == None:
        log.error("No pom.xml path specified")
        sys.exit(1)
        
    if len(pomXmlPaths) > 1:
        log.error("Multiple pom.xml paths specified, only one allowed currently (may change in the future).")
        sys.exit(1)

    pomXmlPath = "./pom.xml"
    if len(pomXmlPaths) > 0:
        pomXmlPath = pomXmlPaths[0]

    if not os.path.exists(pomXmlPath):
        log.error("Specified pom.xml path not found: %s" % pomXmlPath)
        sys.exit(1)

    pomXml = dto.PomXml(location=pomXmlPath, pbaConfiguration=pbaConfiguration)
    pomXml.parsePomXml(pomXmlPath)

    if printModulesOnly:
        if pomXml.modulesPomXmls == None:
            log.error("pom.xml '%s' does not specify modules or module's pom not found, unable to honour '-m'" % pomXmlPath)
            sys.exit(1)
        
    if printParentOnly:
        if pomXml.parentPomXml == None:
            log.error("pom.xml '%s' does not specify full parent coordinates, unable to honour '-p'" % pomXmlPath)
            sys.exit(1)
    
    if listFormat == "csv":
        if printModulesOnly:
            desc = ""
            for modulePomXml in pomXml.modulesPomXmls:
                desc += modulePomXml.describeCoordinates(descFormat=listFormat)
            print(desc)
            sys.exit(0)
        if printParentOnly:
            print(pomXml.parentPomXml.describeCoordinates(descFormat=listFormat))
            sys.exit(0)
        if printMainOnly:
            print(pomXml.describeCoordinates(descFormat=listFormat))
            sys.exit(0)
        print(pomXml.describeCoordinates(descFormat=listFormat))
        sys.exit(0)

    if printParentOnly:
        print(pomXml.parentPomXml.describeCoordinates(descFormat=listFormat))
        sys.exit(0)

    if printMainOnly:
        print(pomXml.describeCoordinates(descFormat=listFormat))
        sys.exit(0)

    #if listFormat == 'text':
    #    print("\nMaven coordinates of %s:\n" % pomXmlPath)
    print(pomXml.describeCoordinates(descFormat=listFormat))
    if pomXml.parentPomXml == None:
        if listFormat == 'text':
            print("\nNo parent coordinates in %s.\n" % pomXmlPath)
    else:
        print(pomXml.parentPomXml.describeCoordinates(descFormat=listFormat))

    if pomXml.modulesPomXmls == None:
        if listFormat == 'text':
            print("\nNo child modules in %s.\n" % pomXmlPath)
    else:
        for modulePomXml in pomXml.modulesPomXmls:
            print(modulePomXml.describeCoordinates(descFormat=listFormat))

    sys.exit(0)
